<?php
/*--------------------------------------------------------------------------------------------------
    ThemeConfigurationFactory.php 2019-10-24
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2019 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Components\Theme\Factories;

use Exception;
use FileNotFoundException;
use FilesystemAdapter;
use Gambio\StyleEdit\Configurations\ShopBaseUrl;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\ActiveThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Translators\ImagePathTranslator;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Translators\LanguageTranslator;
use Gambio\StyleEdit\Core\Components\Theme\Validator;
use Gambio\StyleEdit\Core\Components\Variant\Repositories\VariantRepository;
use Gambio\StyleEdit\Core\Components\Variant\Services\VariantService;
use Gambio\StyleEdit\Core\Language\Services\LanguageService;
use Gambio\StyleEdit\Core\Options\Entities\ConfigurationCategory;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use ReflectionException;
use stdClass;

/**
 * Class ThemeConfigurationFactory
 * @package Gambio\StyleEdit\Core\Components\Theme\Factories
 */
class ThemeConfigurationFactory
{
    /**
     * @var bool
     */
    protected $activeTheme;
    /**
     * @var ShopBaseUrl
     */
    protected $baseUrl;
    /**
     * @var FilesystemAdapter|null
     */
    protected $filesystemThemes;
    /**
     * @var ImagePathTranslator
     */
    protected $imagePathTranslator;
    /**
     * @var LanguageService
     */
    protected $languageService;
    /**
     * @var LanguageTranslator
     */
    protected $languageTranslator;
    /**
     * @var StyleEditConfiguration|null
     */
    protected $styleEditConfiguration;
    
    
    /**
     * ThemeConfigurationFactory constructor.
     *
     * @param ShopBaseUrl                 $baseUrl
     * @param LanguageTranslator          $themeTranslator
     *
     * @param LanguageService             $languageService
     * @param ImagePathTranslator         $imagePathTranslator
     * @param StyleEditConfiguration|null $styleEditConfiguration
     * @param FilesystemAdapter|null      $adapter
     * @param ActiveThemeInterface        $activeTheme
     */
    public function __construct(
        ShopBaseUrl $baseUrl,
        LanguageTranslator $themeTranslator,
        LanguageService $languageService,
        ImagePathTranslator $imagePathTranslator,
        StyleEditConfiguration $styleEditConfiguration,
        FilesystemAdapter $adapter,
        ActiveThemeInterface $activeTheme
    
    ) {
        $this->baseUrl                 = $baseUrl;
        $this->languageTranslator      = $themeTranslator;
        $this->imagePathTranslator     = $imagePathTranslator;
        $this->filesystemThemes        = $adapter;
        $this->styleEditConfiguration  = $styleEditConfiguration;
        $this->languageService         = $languageService;
        $this->activeTheme             = $activeTheme;
    }
    
    
    /** Creates a themeConfiguration instance with all the related data
     *
     * @param stdClass $themeConfig
     *
     * @return ThemeConfiguration
     * @throws FileNotFoundException
     * @throws ReflectionException
     */
    public function createExtendedFromJson(
        stdClass $themeConfig
    ): ThemeConfiguration {
        $this->languageTranslator->translateContent($themeConfig);
        $this->imagePathTranslator->translateContent($themeConfig);
        $configurationRepository = ConfigurationRepository::createForTheme($themeConfig->id);
        $variantService = VariantService::createForTheme($themeConfig->id);

        
        $path                          = $this->styleEditConfiguration->themesFolderPath() . $themeConfig->id;
        
        return new ThemeConfiguration($themeConfig->id ?? null,
                                      $this->getTitle($themeConfig),
                                      $this->getThumbnail($themeConfig),
                                      $this->getAuthor($themeConfig),
                                      $this->getVersion($themeConfig),
                                      $themeConfig->extends ?? null,
                                      $themeConfig->inherits ?? null,
            //07
                                      $this->getIsPreview($themeConfig),
                                      $this->getIsEditable($themeConfig),
                                      $this->getIsRemovable($themeConfig),
                                      $this->getIsActive($themeConfig),
                                      $themeConfig->colorPalette ?? [],
                                      $this->getAreas($themeConfig, $configurationRepository, $variantService),
                                      $this->getBasics($themeConfig, $configurationRepository, $variantService),
                                      $this->getStyles($themeConfig, $configurationRepository, $variantService),
                                      $this->getChildren(),
                                      $this->languageService->getActiveLanguages()->getArray(),
                                      $path);
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return string
     */
    private function getTitle(stdClass $themeConfig): string
    {
        if (isset($themeConfig->id)) {
            return isset($themeConfig->title) ? (string)$themeConfig->title : (string)$themeConfig->id;
        }
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return string|null
     */
    private function getThumbnail(stdClass $themeConfig): ?string
    {
        if (isset($themeConfig->thumbnail)) {
            return (string)$themeConfig->thumbnail;
        }
        
        return null;
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return mixed
     */
    private function getAuthor(stdClass $themeConfig): ?string
    {
        if (isset($themeConfig->author)) {
            return (string)$themeConfig->author;
        }
        
        return null;
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return mixed
     */
    private function getVersion(stdClass $themeConfig): ?string
    {
        if (isset($themeConfig->version)) {
            return (string)$themeConfig->version;
        }
        
        return null;
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return bool
     */
    private function getIsPreview(stdClass $themeConfig): bool
    {
        return isset($themeConfig->preview, $themeConfig->id)
               && $themeConfig->preview
               && strpos($themeConfig->id, '_preview');
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return bool
     * @throws FileNotFoundException
     */
    private function getIsEditable(stdClass $themeConfig): bool
    {
        
        return !$this->getIsPreview($themeConfig)
               && isset($themeConfig->id)
               && Validator::for($themeConfig->id)->canBeOpenedInStyleEdit4();
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return bool
     * @throws Exception
     */
    private function getIsRemovable(stdClass $themeConfig): bool
    {
        
        return !(in_array($themeConfig->id ?? '', $this->getGambioThemeIds(), true)
                 || $this->getIsActive($themeConfig))
               || $this->getIsPreview($themeConfig);
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @return bool
     * @throws Exception
     */
    private function getIsActive(stdClass $themeConfig): bool
    {
        return isset($themeConfig->id) && $themeConfig->id === $this->activeTheme->value();
    }
    
    
    /**
     * @param stdClass $themeConfig
     *
     * @param          $configurationRepository
     * @param          $variantService
     *
     * @return ConfigurationCategory
     * @throws ReflectionException
     */
    private function getAreas(stdClass $themeConfig, ConfigurationRepository $configurationRepository, VariantService $variantService): ?ConfigurationCategory
    {
        if (isset($themeConfig->config, $themeConfig->config->areas) && $themeConfig->config->areas) {
            return ConfigurationCategory::createFromJsonObject($themeConfig->config->areas,
                                                               $variantService,
                                                               $configurationRepository);
        }
        
        return null;
    }
    
    
    /**
     * @param stdClass                $themeConfig
     *
     * @param ConfigurationRepository $configurationRepository
     * @param VariantService          $variantService
     *
     * @return ConfigurationCategory
     * @throws ReflectionException
     */
    private function getBasics(stdClass $themeConfig, ConfigurationRepository $configurationRepository, VariantService $variantService): ?ConfigurationCategory
    {
        if (isset($themeConfig->config, $themeConfig->config->basics) && $themeConfig->config->basics) {
            
            return ConfigurationCategory::createFromJsonObject($themeConfig->config->basics,
                                                               $variantService,
                                                               $configurationRepository);
        }
        
        return null;
    }
    
    
    /**
     * @param stdClass                $themeConfig
     *
     * @param ConfigurationRepository $configurationRepository
     * @param VariantService          $variantService
     *
     * @return ConfigurationCategory
     * @throws ReflectionException
     */
    private function getStyles(stdClass $themeConfig, ConfigurationRepository $configurationRepository, VariantService $variantService): ?ConfigurationCategory
    {
        if (isset($themeConfig->config, $themeConfig->config->styles) && $themeConfig->config->styles) {
            return ConfigurationCategory::createFromJsonObject($themeConfig->config->styles,
                                                               $variantService,
                                                               $configurationRepository);
        }
        
        return null;
    }
    
    
    /**
     * @return array
     */
    protected function getChildren(): array
    {
        return [];
    }
    
    
    /**
     * @return array
     */
    protected function getGambioThemeIds(): array
    {
        return [
            'Honeygrid',
            'Childgrid',
            'Grandgrid'
        ];
    }
    
    
    /** Simplified ThemeConfiguration for listing purposes
     *
     * @param stdClass $themeConfig
     *
     * @return ThemeConfiguration
     * @throws FileNotFoundException
     */
    public function createFromJson(
        stdClass $themeConfig
    ): ThemeConfiguration {
        
        $this->languageTranslator->translateContent($themeConfig);
        $this->imagePathTranslator->translateContent($themeConfig);
        $path = $this->styleEditConfiguration->themesFolderPath() . $themeConfig->id;
        
        return new ThemeConfiguration($themeConfig->id ?? null,//01
                                      $this->getTitle($themeConfig),//02
                                      $this->getThumbnail($themeConfig),//03
                                      $this->getAuthor($themeConfig),//04
                                      $this->getVersion($themeConfig),//05
                                      $themeConfig->extends ?? null,//06
                                      $themeConfig->inherits ?? null,//07
                                      $this->getIsPreview($themeConfig),//08
                                      $this->getIsEditable($themeConfig),//09
                                      $this->getIsRemovable($themeConfig),//10
                                      $this->getIsActive($themeConfig),//11
                                      $themeConfig->colorPalette ?? [],//12
                                      null,//13
                                      null,//14
                                      null,//15
                                      $this->getChildren(),//16
                                      $this->languageService->getActiveLanguages()->getArray(),//17
                                      $path);//18
    }
}