<?php
/*--------------------------------------------------------------------------------------------------
    ConfigurationRepository.php 2019-06-06
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2016 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Repositories;

use Exception;
use Gambio\StyleEdit\Core\BuildStrategies\Interfaces\SingletonStrategyInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Variant\Entities\VariantOption;
use Gambio\StyleEdit\Core\Components\Variant\Json\VariantInheritanceHandler;
use Gambio\StyleEdit\Core\Components\Variant\Services\VariantService;
use Gambio\StyleEdit\Core\Options\Entities\ConfigurationCategory;
use Gambio\StyleEdit\Core\Options\Entities\OptionInterface;
use Gambio\StyleEdit\Core\Repositories\Entities\Configuration;
use Gambio\StyleEdit\Core\Repositories\Entities\ConfigurationCollection;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use stdClass;

/**
 * Class ConfigurationRepository
 * @package Gambio\StyleEdit\Core\Repositories
 */
class ConfigurationRepository extends BasicFileRepository implements SingletonStrategyInterface
{
    /**
     * @var string
     */
    protected $settingsFileName;
    /**
     * @var ConfigurationCollection
     */
    private $configurationList = null;
    /**
     * @var string
     */
    private $themeId;
    
    
    /**
     * ConfigurationRepository constructor.
     *
     * @param  $themeConfig
     */
    public function __construct(CurrentThemeInterface $themeConfig = null)
    {
        if ($themeConfig) {
            $this->themeId = $themeConfig->id();
        }
    }
    
    
    /**
     * @param string $themeId
     */
    protected function setThemeId(string $themeId)
    {
        $this->themeId = $themeId;
    }
    
    
    /**
     * @param string $themeId
     *
     * @return ConfigurationRepository
     */
    public static function createForTheme(string $themeId): self
    {
        $result = new static(null);
        $result->setThemeId($themeId);
        $result->configurationList = null;
        
        return $result;
    }
    
    
    /**
     * @param string $name
     *
     * @return Configuration
     * @throws \Exception
     */
    public function getJsonConfigurationFrom(string $name)
    {
        // Transform camelCase to dash-case because the settings.json is dash-cased and theme.json is camelCased
        $name = strtolower(preg_replace('/([a-zA-Z])(?=[A-Z])/', '$1-', $name));
        
        if ($this->configurationsList()->keyExists($name)) {
            return $this->configurationsList()->getValue($name);
        }
        
        return false;
    }
    
    
    /**
     * @return ConfigurationCollection
     * @throws \Exception
     */
    public function configurationsList(): ConfigurationCollection
    {
        if ($this->configurationList === null) {
            $this->configurationList = ConfigurationCollection::createFromJsonList($this->loadConfigurationsObject());
        }
        
        return $this->configurationList;
    }
    
    
    /**
     * @return array|mixed
     * @throws Exception
     */
    private function loadConfigurationsObject()
    {
        return $this->loadJsonFilesFromDisk($this->getSettingsFilename());
    }
    
    
    /**
     * @return string
     * @throws Exception
     */
    protected function getSettingsFilename(): string
    {
        if ($this->settingsFileName === null) {
            $configurations         = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
            $this->settingsFileName = $configurations->themesFolderPath() . $this->themeId . DIRECTORY_SEPARATOR
                                      . 'settings.json';
        }
        
        return $this->settingsFileName;
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Repositories\Entities\Configuration $configuration
     *
     * @throws \Exception
     */
    public function saveJsonConfigurationFrom(Configuration $configuration)
    {
        $this->configurationsList()->setValue($configuration->id(), $configuration);
        $this->configurationsList()->sort();
        $this->saveJsonFilesToDisk($this->configurationsList(), $this->getSettingsFilename());
    }
    
    
    /**
     * @param \Gambio\StyleEdit\Core\Options\Entities\OptionInterface $option
     *
     * @throws \Exception
     */
    public function saveOptionToConfiguration(OptionInterface $option): void
    {
        if ($this->configurationsList()->keyExists($option->id())) {
            $jsonConfiguration = $this->configurationsList()->getValue($option->id());
            $jsonConfiguration = $jsonConfiguration->jsonSerialize();
            if (!isset($jsonConfiguration->type)) {
                $jsonConfiguration->type = $option->type();
            }
        } else {
            $jsonConfiguration        = new stdClass();
            $jsonConfiguration->name  = $option->id();
            $jsonConfiguration->type  = $option->type();
            $jsonConfiguration->group = $option->group() ?? 'bootstrap';
        }
        
        $jsonConfiguration->value = $option->value();
        
        $configuration = Configuration::createFromJson($jsonConfiguration);
        $this->configurationsList()->setValue($configuration->id(), $configuration);
        if ($option->type() === "variant") {
            $this->loadVariantSettings($option);
        }
        $this->saveJsonFilesToDisk($this->configurationsList(), $this->getSettingsFilename());
        
        // variant specific option
        if ($option->for() !== null) {
            $this->saveConfigurationToVariant($configuration, $option->for());
        }
    }
    
    
    /**
     * @param Configuration $configuration
     *
     * @param string        $variant
     *
     * @throws Exception
     */
    protected function saveConfigurationToVariant(Configuration $configuration, string $variant): void
    {
        [$variantId, $variantOptionId] = explode('/', $variant);
        $variantSettingsFilename = $this->variantSettingsFilename($variantId, $variantOptionId);
        
        try {
            $variantJson = $this->loadJsonFilesFromDisk($variantSettingsFilename);
        } catch (Exception $exception) { // if file was not found
            $variantJson = [];
        }
        
        $variantConfiguration = ConfigurationCollection::createFromJsonList($variantJson);
        $variantConfiguration->setValue($configuration->id(), $configuration);
        $this->saveJsonFilesToDisk($variantConfiguration, $variantSettingsFilename);
    }
    
    
    /**
     * @param string $variantId
     * @param string $variantOptionId
     *
     * @return string
     * @throws Exception
     */
    protected function variantSettingsFilename(string $variantId, string $variantOptionId): string
    {
        $configurations = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
        
        $variantsDirectoryPath = $configurations->themesFolderPath() . $this->themeId . DIRECTORY_SEPARATOR
                                 . 'variants';
        
        return implode(DIRECTORY_SEPARATOR, [$variantsDirectoryPath, $variantId, $variantOptionId, 'settings.json']);
    }
    
    
    /**
     * @param VariantOption $option
     *
     * @throws Exception
     */
    public function restoreVariantSettings(VariantOption $option): void
    {
        if (!empty($option->id()) && !empty($option->value()->id())) {
            
            $this->initializeVariantSettings($option);
            
            $variantId            = $option->id();
            $variantOptionId      = $option->value()->id();
            $variantSettingsFile  = $this->variantSettingsFilename($variantId, $variantOptionId);
            $variantSettingsJson  = $this->loadJsonFilesFromDisk($variantSettingsFile);
            $variantConfiguration = ConfigurationCollection::createFromJsonList($variantSettingsJson);
            
            if ($variantConfiguration->count()) {
                foreach ($variantConfiguration as $variantOption) {
                    $this->configurationsList()->setValue($variantOption->id(), $variantOption);
                }
                $this->saveJsonFilesToDisk($this->configurationsList(), $this->getSettingsFilename());
            }
        }
    }
    
    /**
     * @param VariantOption $option
     *
     * @throws Exception
     */
    protected function initializeVariantSettings(VariantOption $option): void
    {
        $variantSettingsFile = $this->variantSettingsFilename($option->id(), $option->value()->id());
        $variantFile = dirname($variantSettingsFile).DIRECTORY_SEPARATOR.'variant.json';
        
        try {
            $configurationRepository = self::createForFile($variantSettingsFile);
    
            /**
             * @var $handler VariantInheritanceHandler
             */
            $handler = SingletonPrototype::instance()->get(VariantInheritanceHandler::class);
            $handler->setFilename($variantFile);
            $variantJson = $handler->execute();
            // Copies the default values to settings.json
            ConfigurationCategory::createFromJsonObject($variantJson,
                                                        SingletonPrototype::instance()->get(VariantService::class),
                                                        $configurationRepository);
        } catch (Exception $exception) {
        }
    }
    
    
    /**
     * @param string $variantId
     * @param string $variantOptionId
     *
     * @return string
     * @throws Exception
     */
    private function variantFileName(string $variantId, string $variantOptionId): string
    {
        $configurations = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
        
        $variantsDirectoryPath = $configurations->themesFolderPath() . $this->themeId . DIRECTORY_SEPARATOR
                                 . 'variants';
        
        return $variantsDirectoryPath . DIRECTORY_SEPARATOR . $variantId . DIRECTORY_SEPARATOR . $variantOptionId
               . DIRECTORY_SEPARATOR . 'variant.json';
    }
    
    
    /**
     * @param $filename
     *
     * @return static
     * @throws Exception
     */
    public static function createForFile($filename): ConfigurationRepository
    {
        $result                   = new static(null);
        $result->settingsFileName = $filename;
        $result->configurationList = null;
        
        if (!$result->fileIO()->exists($filename)) {
            $result->fileIO()->write([], $filename);
        }
        
        return $result;
    }
    
    
    /**
     * @return array|mixed
     * @throws Exception
     */
    public function hasConfiguration()
    {
        return $this->fileIO()->exists($this->getSettingsFilename());
    }
    
    protected function loadVariantSettings(OptionInterface $option)
    {
        $variantSettingsPath = implode(DIRECTORY_SEPARATOR, [
            $this->configuration()->themesFolderPath(),
            $this->themeId,
            'variants',
            $option->id(),
            $option->value()->id(),
            'settings.json'
        ]);
        
        if($this->fileIO->exists($variantSettingsPath)){
            $jsonArray = $this->fileIO->read($variantSettingsPath);
            foreach ($jsonArray as $configuration) {
                $configuration = Configuration::createFromJson($configuration);
                $this->configurationsList()->setValue($configuration->id(), $configuration);
            }
        }
    }
}